#!/bin/sh
#
# build-arm-uclinux.sh - a script for building arm-uclinux cross toolchain
#
# This script should have come in a tar file: tool-build-script.tar.gz
#  along with 4 other files:
#        README                - Instructions
#        bin2asm.Makefile.in   - a replacement Makefile.in
#        clean-prefix.sh       - a clean utility
#        COPYING               - The GNU General Public License
#
# Please see README and COPYING for further details.
#
# John E. Andrews - john@triscend.com , johne.andrews@home.com
#
# all other files downloaded from ftp://ftp.wirespeed.com/uclinux
#
#	arm_uclinux_binutils_10_31_2000.tar.gz
#	arm_uclinux_gcc_10_31_2000.tar.gz
#	uC-glibc-2.1.3-10112000.tar.gz
#	ldelf2flt-12192000.tar.gz
#	bin2asm-05122000.tar.gz
#
# NOTE: replacement Makefile.in for bin2asm - bin2asm.Makefile.in
#       must also be present in download directory
#
# NOTE: Do NOT set PREFIX=/usr
#       This will overwrite native tools!
#
# -jea 6/12/2001
#
# Set KERNEL_SRC to the top level uClinux source directory
export KERNEL_SRC=/home/johnea/TA7Lx/kernel/linux
# Set PREFIX to the desired install directory (See NOTE above!)
export PREFIX=/usr/local/wirespeed

echo "#########################################################"
echo "#         Copyright (C) 2001 John E. Andrews"
echo "#         see README and COPYING for details"
echo "#"
echo "# Building arm-uclinux crosstools in " `pwd`
echo "#  Install Prefix = ${PREFIX}"
echo "#  Kernel Source = ${KERNEL_SRC}"
echo "#"
echo "# Copying uClinux headers"
echo "#########################################################"
echo "#"

# make dir and copy kernel header files into install dir tree
mkdir -p ${PREFIX}/arm-uclinux/include 
cp -r ${KERNEL_SRC}/include/linux ${PREFIX}/arm-uclinux/include
cp -r ${KERNEL_SRC}/include/asm-armnommu ${PREFIX}/arm-uclinux/include/asm

# configure, compile and install binutils
echo "#########################################################"
echo "#"
echo "# Unpacking binutils source"
echo "#"
echo "#########################################################"
tar xzf arm_uclinux_binutils_10_31_2000.tar.gz
mkdir build-binutils
cd build-binutils

# configure
echo "#"
echo "# Configuring binutils in " `pwd`
echo "#"
echo "#########################################################"
../arm_uclinux_binutils/configure --target=arm-uclinux --prefix=$PREFIX 2>&1 | tee configure.log

# compile
echo "#########################################################"
echo "#"
echo "# Compiling binutils"
echo "#"
echo "#########################################################"
make all 2>&1 | tee make.log
# TODO: make error checking doesn't work
if [ $? != 0 ]; then
    echo "Compiling binutils failed." && exit 1;
fi

# install
echo "#########################################################"
echo "#"
echo "# Installing binutils"
echo "#"
echo "#########################################################"
make install 2>&1 | tee install.log
cd ..

# setup path for cross tool binaries
export PATH=${PREFIX}/bin:${PATH}

# make a bootstrap gcc with no lib support
echo "#########################################################"
echo "#"
echo "# Unpacking bootstrap gcc source"
echo "#"
echo "#########################################################"
tar xzf arm_uclinux_gcc_10_31_2000.tar.gz
mkdir build-boot-gcc
cd build-boot-gcc

## configure
echo "#"
echo "# Configuring bootstrap gcc in " `pwd`
echo "#"
echo "#########################################################"
../arm_uclinux_gcc/configure --target=arm-uclinux --prefix=$PREFIX --with-local-prefix=${PREFIX}/arm-uclinux --enable-languages=c 2>&1 | tee boot-configure.log

# compile with CFLAGS+=-Dinhibit_libc to avoid uninstalled headers
echo "#########################################################"
echo "#"
echo "# Compiling bootstrap gcc"
echo "#"
echo "#########################################################"
make all-gcc CFLAGS+=-Dinhibit_libc 2>&1 | tee boot-make.log
# TODO: make error checking doesn't work
if [ $? != 0 ]; then
    echo "Compiling bootstrap gcc failed." && exit 1;
fi

echo "#########################################################"
echo "#"
echo "# Installing bootstrap gcc"
echo "#"
echo "#########################################################"
# install
make install-gcc 2>&1 | tee boot-install.log
cd ..

# config, make and install glibc
echo "#########################################################"
echo "#"
echo "# Unpacking glibc source"
echo "#"
echo "#########################################################"
tar xzf uC-glibc-2.1.3-10112000.tar.gz
mkdir build-glibc
cd build-glibc

# set environment to build glibc w/ cross tools
export CC=arm-uclinux-gcc
export AR=arm-uclinux-ar
export RANLIB=arm-uclinux-ranlib

# configure
echo "#"
echo "# Configuring glibc in " `pwd`
echo "#"
echo "#########################################################"
../glibc-2.1.3-uclinux-arm/configure --host=arm-uclinux --enable-add-ons --prefix=${PREFIX}/arm-uclinux --with-headers=${PREFIX}/arm-uclinux/include --without-fp --program-prefix="arm-uclinux-" --disable-profile --enable-omit-fp --disable-shared --enable-static-nss --disable-libio  2>&1 | tee configure.log
# TODO: make error checking doesn't work
if [ $? != 0 ]; then
    echo "Configuring glibc failed." && exit 1;
fi

# compile
echo "#########################################################"
echo "#"
echo "# Compiling glibc"
echo "#"
echo "#########################################################"
make all 2>&1 | tee make.log
# TODO: make error checking doesn't work
if [ $? != 0 ]; then
    echo "Compiling glibc failed." && exit 1;
fi

# install
echo "#########################################################"
echo "#"
echo "# Installing glibc"
echo "#"
echo "#########################################################"
make install 2>&1 | tee install.log
cd ..

# cleanup, recompile and install a full gcc now that glibc headers are installed
echo "#########################################################"
echo "#"
echo "# Unpacking gcc source"
echo "#"
echo "#########################################################"
tar xzf arm_uclinux_gcc_10_31_2000.tar.gz
mkdir build-gcc
cd build-gcc

# set environement to build crossgcc w/ native tools
export CC=gcc
export AR=ar
export RANLIB=ranlib

# configure
echo "#"
echo "# Configuring gcc in " `pwd`
echo "#"
echo "#########################################################"
../arm_uclinux_gcc/configure --target=arm-uclinux --prefix=$PREFIX --with-local-prefix=${PREFIX}/arm-uclinux --enable-languages=c,c++ 2>&1 | tee configure.log
# TODO: make error checking doesn't work
if [ $? != 0 ]; then
    echo "Configuring gcc failed." && exit 1;
fi

# compile
echo "#########################################################"
echo "#"
echo "# Compiling gcc"
echo "#"
echo "#########################################################"
make all 2>&1 | tee make.log
# TODO: make error checking doesn't work
if [ $? != 0 ]; then
    echo "Compiling gcc failed." && exit 1;
fi

# install
echo "#########################################################"
echo "#"
echo "# Installing gcc"
echo "#"
echo "#########################################################"
make install 2>&1 | tee install.log
cd ..


# configure, compile and install ldelf2flt
echo "#########################################################"
echo "#"
echo "# Unpacking ldelf2flt source"
echo "#"
echo "#########################################################"
tar xzf ldelf2flt-12192000.tar.gz
mkdir build-ldelf2flt
cd build-ldelf2flt
# There seems to be a typo in the tarball's Makefile.in: change -i to -I
mv ../ldelf2flt/Makefile.in \
	../ldelf2flt/Makefile.in-original
sed "s/CFLAGS = -O2 -g -DDEBUG -Wall -i/CFLAGS = -O2 -g -DDEBUG -Wall -I/" \
	< ../ldelf2flt/Makefile.in-original \
	> ../ldelf2flt/Makefile.in

# configure
echo "#"
echo "# Configuring ldelf2flt in " `pwd`
echo "#"
echo "#########################################################"
../ldelf2flt/configure --prefix=$PREFIX

# compile 
echo "#########################################################"
echo "#"
echo "# Compiling ldelf2flt"
echo "#"
echo "#########################################################"
make

# install
echo "#########################################################"
echo "#"
echo "# Installing ldelf2flt"
echo "#"
echo "#########################################################"
make install
cd ..

# configure, compile and install bin2asm
echo "#########################################################"
echo "#"
echo "# Unpacking bin2asm source"
echo "#"
echo "#########################################################"
tar xzf bin2asm-05122000.tar.gz
mkdir build-bin2asm
cd build-bin2asm

# The original Makefile.in has no build rules: replace with update
mv ../bin2asm/Makefile.in ../bin2asm/Makefile.in.org
cp ../bin2asm.Makefile.in ../bin2asm/Makefile.in

# configure
echo "#"
echo "# Configuring bin2asm in " `pwd`
echo "#"
echo "#########################################################"
../bin2asm/configure --prefix=$PREFIX

# compile 
echo "#########################################################"
echo "#"
echo "# Compiling bin2asm"
echo "#"
echo "#########################################################"
make

# install
echo "#########################################################"
echo "#"
echo "# Installing bin2asm"
echo "#"
echo "#########################################################"
make install
cd ..

echo "#########################################################"
echo "#"
echo "# Build Complete: The arm-uclinux cross toolchain"
echo "#  is ready to build your kernel and apps!"
echo "#"
echo "#########################################################"

